# 合约说明

仅采用公用的合约特性来编写合约，可以完成任务需求。故合约未采用fisco-bcos提供的weEvent特性等机制，保证合约能够在使用solidiy合约的链上运行。

经验证，本合约能够在fisco-bcos、cita上进行运行，并将该合约进行了向fabric的链码进行转写。

相关工程代码：

- [bsn](https://github.com/chislab/bsn.git) 合约开发相关
- [bsn_backend](https://github.com/chislab/bsn_backend.git)  后端验证简易Server

## AccessControl 授权合约

本合约用于各种合约间角色的约定及调用权限。比如普通用户没有权限去向供货商下订单，而供货商也不可能向普通用户提供生产完成的商品。

```
// address是否有role权限
function hasRole(bytes32 role, address account) public view returns (bool) 

// 获取role权限的成员数
function getRoleMemberCount(bytes32 role) public view returns (uint256)

// 根据role和index获取成员
function getRoleMember(bytes32 role, uint256 index) public view returns (address)

// 获取role角色的管理者
function getRoleAdmin(bytes32 role) public view returns (bytes32)

// 向某个account授予role权限
function grantRole(bytes32 role, address account) public virtual

// 移除account的role权限
function revokeRole(bytes32 role, address account) public virtual

// account放弃持有的role权限
function revokeRole(bytes32 role, address account) public virtual
```

## Produce 产品合约

本合约用于完成产品生产过程中产品注册及产品所有权的变更，及产品溯源。

步骤如下:

- 完成对相关合约的关系设置
- 合约的admin向某些address授权成为product生产商
- product生产商调用updateProductPrice对产品价格进行设定
- customer需要向生产商下产品生产订单前会先getProductPrice来比对价格的高低，然后通过payment合约向生产商下产品生产订单
- 生产商通过payment合约获取到订单后，进行产品生产
- 在生产产品前，生产商会通过Material合约来检查自己的产品原料储备是否充足，如果充足就消耗原料进行产品的生产注册registerProduct上链。
- 产品生产商进行交付，customer确认收到货品，确认订单，完成资金及产品所有权的变更。

```
// 厂商msg.sender更新自己productType产品的价格
function updateProductPrice(uint256 productType, uint256 newPrice) public

// 获取厂商 _to的productType的价格
function getProductPrice(address _to, uint256 productType) public view returns(uint256 price)

// 设置供应商的原材料合约
function setMaterialContract(address _materialContract) public onlyOwner

// 设置支付合约
function setPaymentContract(address _paymentContract) public onlyOwner

// 产品生产商设置自己产品的批次号，ID，所用材料批次等信息
function registerProduct(uint256 productType, uint256 id, uint256 batchNumber, uint256[] memory materialBatches) public

// 更换产品所有权，交付产品
function transferProducts(address from, address to, uint256 productType, uint256 count) public

// 获取产品详情
function details(uint256 id) public view returns(Product memory)

// 获取msg.sender所拥有的产品数组
function getMyProducts(uint256 productType) public view returns(uint256[] memory myProductIDs)

// 获取某种产品的原材料用于哪些产品了
function trace(uint256 materialBatchNum) public view returns(uint256[] memory ids)
```

## Material 供应商合约

本合约预先假定了某种材料只有一个供应商，本合约作为授权

调用流程大致如下:

- 完成对相关合约的关系设置
- Material合约的admin授权某些address成为Material生产商
- Material生产商设置Material价格
- 产品生产商向Material生产商通过Payment合约下原材料订单
- 如果Material生产商所持有的数目够，可以把多个批次的零件一次性进行transferMaterial交付
- 如果不足以交付则通过newMaterial进行零件注册上链
- 产品生产商收到零件后生产产品，通过consumeMaterial进行零件的消耗登记
- getMyMaterial， showBatchInfo是一些查询相关信息的函数

```
// 更新某处原材料的供应商
function updateProducer(uint256 materialType, address producer) public

// 对原材料进行注册上链，如材料类型，总量，批次号
function newMaterial(uint256 materialType, uint256 totalNum, uint256 batchID) public returns(RawMaterial memory m)

// 获取材料的生产商
function getMaterialProducer(uint256 materialType) public view returns(address)

// 完成原材料的交付，如果某批次材料不足，一次可以交付多个批次，直到数量满足
function transferMaterial(address _from, address _to, uint256 materialType, uint256 num) public

// 生产产品中消耗原材料登记
function consumeMaterial(uint256 materialType, uint256 num) public

// 获取自己所拥有的某种材料的数目
function getMyMaterial(uint256 materialType) public view returns (uint256 num)

// 根据材料的批次ID获取批次信息
function showBatchInfo(uint256 batchID) public view
    returns(address producer, uint256 createdAt, uint256 batchId, uint256 materialType, uint256 totalNum, uint256 keptNum)
    
// 获取原材料价格
function getPrice(address to, uint256 materialType) public view returns(uint256)

// 设置原材料价格
function setPrice(uint256 materialType, uint256 price) public
```

## Payment支付合约

用于生产过程中的原材料、产品的金融流动，负责订单的生成，确认，取消；资金的流转，销毁。在peyer下订单前，会对余额进行检查。如果余额不足，可以通过Payment的admin来增加余额。

- makeOrder，payer向订单收到方下订单，生产方在收到订单后，会进行订单的确认或取消，决定自己的生产计划
- cancelOrder，按取消方的角色定位，返还约定好的资金比例
- getOrder 根据订单id获取订单详情
- confirmOrder payer在确认收到对方运输的货物(不论是材料还是生产的产品)后，需进行订单的确认，才能调用Material合约或者Product合约的tranfer函数，转移原材料或者产品的所有权，生产方拿到货款。

```
// 设置供应商的材料合约
function setMaterialProducer(address producer) public onlyOwner

// 设置生产商合约
function setProductProducer(address producer) public onlyOwner

// 下生产订单，isMaterial是产品or材料，to订单收到方，orderType 产品or材料类型，_count数目，_price价格
function makeOrder (bool isMaterial, address to, uint256 orderType, uint256 _count, uint256 _price) public returns(uint256 id)

// 确认订单已经完成, Payer确认后，资金划归到订单收到方
function confirmOrder(uint256 id) public

// 取消订单，订单取消，按订单的取消方不同，对金额按照不同比例进行划归
function cancelOrder(uint256 id) public

// 根据订单id察看订单详情
function getOrder(uint256 id) public view returns(Order memory order)

// 销毁资金
function burn(address account, uint256 amount) public onlyOwner

// 充值
function mint(address account, uint256 amount) public onlyOwner

// 获取余额
function balanceOf(address account) public view returns(uint256 amount)
```

## 附 开发心得

开发过程中，最主要是对整个生产流程的梳理，针对各个链的开发，或者有自己特有的合约语言(如Fabric的链码)其实并不是很难，只是对过程的一种描述。

因此，整个开发流程，我们确立了先梳理需求，然后针对solidity语言先开发出一个可用的版本，再在此基础上对相关调用建立一个简单的restful服务器，完成对各个接口的测试，有一套完整的可演示流程。最后针对各个链进行相关开发完成对合约的复现。

